//###########################################################################
// FILE:    Example_28X7xSci_Echoback.c
// TITLE:   SCI Echoback.
//
//! \addtogroup cpu01_example_list
//! <h1>SCI Echoback</h1>
//!
//!  This test receives and echo-backs data through the SCI-A port.
//!
//!  The PC application 'hypterterminal' can be used to view the data
//!  from the SCI and to send information to the SCI.  Characters received
//!  by the SCI port are sent back to the host.
//!
//!  \b Running \b the \b Application
//!  -# Configure hyperterminal:
//!  Use the included hyperterminal configuration file SCI_96.ht.
//!  To load this configuration in hyperterminal
//!    -# Open hyperterminal
//!    -# Go to file->open
//!    -# Browse to the location of the project and
//!       select the SCI_96.ht file.
//!  -# Check the COM port.
//!  The configuration file is currently setup for COM1.
//!  If this is not correct, disconnect (Call->Disconnect)
//!  Open the File-Properties dialog and select the correct COM port.
//!  -# Connect hyperterminal Call->Call
//!  and then start the 2806x SCI echoback program execution.
//!  -# The program will print out a greeting and then ask you to
//!  enter a character which it will echo back to hyperterminal.
//!
//!  \note If you are unable to open the .ht file, you can create
//!  a new one with the following settings
//!  -  Find correct COM port
//!  -  Bits per second = 9600
//!  -  Date Bits = 8
//!  -  Parity = None
//!  -  Stop Bits = 1
//!  -  Hardware Control = None
//!
//!  \b Watch \b Variables \n
//!  - \b LoopCount, for the number of characters sent
//!  - ErrorCount
//!
//! \b External \b Connections \n
//!  Connect the SCI-A port to a PC via a transceiver and cable.
//!  - GPIO28 is SCI_A-RXD (Connect to Pin3, PC-TX, of serial DB9 cable)
//!  - GPIO29 is SCI_A-TXD (Connect to Pin2, PC-RX, of serial DB9 cable)
//
//###########################################################################
// $TI Release: F2837xD Support Library v110 $
// $Release Date: Mon Apr  7 10:35:37 CDT 2014 $
//###########################################################################

#include "F28x_Project.h"     // Device Headerfile and Examples Include File

// Prototype statements for functions found within this file.
void scia_echoback_init(void);
void scia_fifo_init(void);
void scia_xmit(int a);
void scia_msg(char *msg);

// Global counts used in this example
Uint16 LoopCount;
Uint16 ErrorCount;

void main(void)
{

    Uint16 ReceivedChar;
    char *msg;

// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the F2837xD_SysCtrl.c file.
   InitSysCtrl();

// Step 2. Initialize GPIO:
// This example function is found in the F2837xD_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
   InitGpio();

// For this example, only init the pins for the SCI-A port.
// These functions are found in the F2837xD_Gpio.c file.
   GPIO_SetupPinMux(57, GPIO_MUX_CPU1, 6);
   GPIO_SetupPinOptions(57, GPIO_INPUT, GPIO_ASYNC);
   GPIO_SetupPinMux(56, GPIO_MUX_CPU1, 6);
   GPIO_SetupPinOptions(56, GPIO_OUTPUT, GPIO_ASYNC);


// Step 3. Clear all __interrupts and initialize PIE vector table:
// Disable CPU __interrupts
   DINT;

// Initialize PIE control registers to their default state.
// The default state is all PIE __interrupts disabled and flags
// are cleared.
// This function is found in the F2837xD_PieCtrl.c file.
   InitPieCtrl();

// Disable CPU __interrupts and clear all CPU __interrupt flags:
   IER = 0x0000;
   IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the __interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in F2837xD_DefaultIsr.c.
// This function is found in F2837xD_PieVect.c.
   InitPieVectTable();

// Step 4. Initialize all the Device Peripherals:
// This function is found in F2837xD_InitPeripherals.c
// InitPeripherals(); // Not required for this example

// Step 5. User specific code:

    LoopCount = 0;
    ErrorCount = 0;
    EALLOW;
    CpuSysRegs.PCLKCR7.bit.SCI_C = 1;
    scia_fifo_init();	   // Initialize the SCI FIFO
    scia_echoback_init();  // Initialize SCI for echoback
    EDIS;
    //GpioDataRegs.GPBSET.bit.GPIO55=1;

    msg = "\r\n\n\nHello World!\0";
    scia_msg(msg);

    msg = "\r\nYou will enter a character, and the DSP will echo it back! \n\0";
    scia_msg(msg);

	for(;;)
    {  int i=0;
       msg = "\r\nEnter a character: \0";
       scia_msg(msg);
       //GpioDataRegs.GPBCLEAR.bit.GPIO56=1;
       GpioDataRegs.GPBSET.bit.GPIO56=1;
       // Wait for inc character

       while(ScicRegs.SCIFFRX.bit.RXFFST ==0) { } // wait for XRDY =1 for empty state

       // Get character
       //ReceivedChar = ScicRegs.SCIRXBUF.all;
       msg = "  You sent: \0";
              scia_msg(msg);
              //scia_xmit(ReceivedChar);
             GpioDataRegs.GPBSET.bit.GPIO56=1;
       do
       	   {

    	   /*for(j=0;j<10000;j++)
    	                {}*/
    	   //GpioDataRegs.GPBCLEAR.bit.GPIO56=1;
                ReceivedChar = ScicRegs.SCIRXBUF.all;


                // Echo character back


                scia_xmit(ReceivedChar);
                GpioDataRegs.GPBSET.bit.GPIO56=1;
             /*for(j=0;j<10000;j++)
             {

             }*/
                i++;

       		}while(ScicRegs.SCIFFRX.bit.RXFFST != 0);/**/
                 LoopCount++;
       	  }

       // Echo character back
       	/* msg = "  You sent: \0";
       scia_msg(msg);
       scia_xmit(ReceivedChar);*/
    //}



}

// Test 1,SCIA  DLB, 8-bit word, baud rate 0x000F, default, 1 STOP bit, no parity
void scia_echoback_init()
{
    // Note: Clocks were turned on to the SCIA peripheral
    // in the InitSysCtrl() function

 	ScicRegs.SCICCR.all =0x0007;   // 1 stop bit,  No loopback
                                   // No parity,8 char bits,
                                   // async mode, idle-line protocol
	ScicRegs.SCICTL1.all =0x0003;  // enable TX, RX, internal SCICLK,
                                   // Disable RX ERR, SLEEP, TXWAKE
	//ScicRegs.SCICTL2.all =0x0003;
	ScicRegs.SCICTL2.bit.TXINTENA =1;
	ScicRegs.SCICTL2.bit.RXBKINTENA =1;
	//ScicRegs.SCICCR.bit.LOOPBKENA =1;
	ScicRegs.SCIHBAUD    =0x0002;  // 9600 baud @LSPCLK = 50MHz (200 MHz SYSCLK).
    ScicRegs.SCILBAUD    =0x008B;

	ScicRegs.SCICTL1.all =0x0023;  // Relinquish SCI from Reset
	ScicRegs.SCIFFTX.bit.TXFIFOXRESET=1;
	   ScicRegs.SCIFFRX.bit.RXFIFORESET=1;
}

// Transmit a character from the SCI
void scia_xmit(int a)
{
    while (ScicRegs.SCIFFTX.bit.TXFFST != 0) {}
    ScicRegs.SCITXBUF=a;

}

void scia_msg(char * msg)
{
    int i;
    i = 0;
    while(msg[i] != '\0')
    {
        scia_xmit(msg[i]);
        i++;
    }
}

// Initialize the SCI FIFO
void scia_fifo_init()
{
	//ScicRegs.SCIFFTX.all=0xC022;
	//ScicRegs.SCIFFRX.all=0x0022;
	ScicRegs.SCIFFTX.all=0xE040;
    ScicRegs.SCIFFRX.all=0x204f;
    ScicRegs.SCIFFCT.all=0x0;

}

//===========================================================================
// No more.
//===========================================================================

